import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import java.io.*;
import java.util.*;

/**
   This class parses TRECVID known-item search submissions 
   using the associated DTDs and outputs a formatted version
   of each submission.

   This software was produced by NIST, an agency of the U.S.
   government, and by statute is not subject to copyright in the
   United ed States. Recipients of this software assume all
   responsibilities associated with its operation, modification and
   maintenance.

*/

public class CheckKisSubmissions{
    
    static PrintWriter writer;


    /**

       To run the program, simply type:
       
       java CheckKisSubmissions
           .../file containing the list of submission files (with full path)
	       .../directory to contain the output tables

       Note that the XML parsing requires these two DTDs:

       - videoSearchResults.dtd
       - videoSearchRunResult.dtd

       Note that XML parsing is memory-intensive, and you may need to
       specify more memory than the default allotment for the JVM.
       You can do this as follows.  For example, to allot 1000M
       (i.e. 1 gigabyte) of memory for running on the feature task, run:

       java -Xmx1000M -Xms1000M FilterFeatureSubmission ....

     */
    public static void main(String[] args) {
	if (args.length < 2)
	    usage();

	String inputFileList = args[0];
	String outputPath = args[1];

	// These are declared out here so that they can be used in error messages
	try {
	    // open file reader
	    BufferedReader reader = new BufferedReader(
					        new FileReader(inputFileList));
	    // create XML parser (dbuilder)
	    DocumentBuilder dbuilder = 
		DocumentBuilderFactory.newInstance().newDocumentBuilder();
	    String nextDoc;
	    Document doc;

	    while((nextDoc = reader.readLine()) != null) {

		System.out.println("Processing " + nextDoc);
		doc=dbuilder.parse(nextDoc);

	        processSearchSubmissionFile(doc,nextDoc,outputPath);
	    }
	    reader.close();
	}
	catch (Exception e) {
	    System.err.println(e);
	    e.printStackTrace();
	    System.exit(-1);
	}
    }


    /** Output program usage message and exit. */
    public static void usage() {
	System.out.println("Usage: java CheckKisSubmission <inputfilelist> <outputpath>");
	System.out.println("<inputfilelist>  -  location of submission.list file");
	System.out.println("<outputpath>     -  location of output file\n");
	System.exit(0);
    }

    /** Process a submission file */
    public static void processSearchSubmissionFile(Document doc, String nextDoc, String outputPath) {
	int runIndex = 0;
	int index = 0;
        Object junk;
	String feat = "";

	index = nextDoc.lastIndexOf("/");
	String fileName = nextDoc.substring(index+1);

	makeNewRunOutputFile(outputPath,fileName);

        writer.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>"); 
        writer.println("<!DOCTYPE videoSearchResults  SYSTEM \"videoSearchResults.dtd\">"); 
	writer.println("<videoSearchResults>");

	try {
	    // get the run result nodes
	    NodeList runs = doc.getElementsByTagName("videoSearchRunResult");

	    // For each video search run result
	    for (int k = 0; k < runs.getLength(); k++) {
		Node currRun = runs.item(k);
		runIndex++;
		NamedNodeMap attributes = currRun.getAttributes();
		String sysid = getAttribute("sysId",attributes,runIndex,null,null,null);
		String priority = getAttribute("priority",attributes,runIndex,sysid,null,null);
		String ptype = getAttribute("pType",attributes,runIndex,sysid,null,null);
		String trtype = getAttribute("trType",attributes,runIndex,sysid,null,null);
		String usersat = getAttribute("usersat",attributes,runIndex,sysid,null,null);
		String desc = getAttribute("desc",attributes,runIndex,sysid,null,null);
		String condition = getAttribute("condition",attributes,runIndex,sysid,null,null);

		writer.println("\n  <videoSearchRunResult ");
		writer.println("    " + "sysId=\""   + sysid+    "\"");
		writer.println("    " + "priority=\""+ priority+ "\"");
		writer.println("    " + "pType=\""+ ptype+ "\"");
		writer.println("    " + "trType=\""  + trtype+   "\"");
		writer.println("    " + "userSat=\""  + usersat+   "\"");
		writer.println("    " + "desc=\""  + desc+   "\"");
		writer.println("    " + "condition=\""  + condition+   "\">");

		// get all the results
		NodeList unordResults = currRun.getChildNodes();
		// and order them
		NodeList results = new OrderedNodeList(unordResults,"tNum");

                // For each feature result
		for (int j = 0; j < results.getLength(); j++) {
		    Node currResult = results.item(j);
		    if (currResult != null && currResult.getAttributes() != null) {
			String topicNum = getAttribute("tNum",currResult.getAttributes(),runIndex,sysid,new Integer(j).toString(),null);
			String elapsedTime = getAttribute("elapsedTime",currResult.getAttributes(),runIndex,sysid,new Integer(j).toString(),null);
			writer.println("\n    "+"<videoSearchTopicResult tNum=" + "\"" + topicNum + "\"" +  " elapsedTime=\"" + elapsedTime + "\"/>");
			// get all the items
			NodeList items = currResult.getChildNodes();

			// For each result item
			for (int i = 0; i < items.getLength(); i++) {
			    Node currItem = items.item(i);
			    if (currItem != null && currItem.getAttributes() != null) {
				attributes = items.item(i).getAttributes();
				String seqNum = getAttribute("seqNum",attributes,runIndex,sysid,topicNum,new Integer(i).toString());
				String shotId = getAttribute("shotId",attributes,runIndex,sysid,topicNum,seqNum);
				writer.println("      " + "<item seqNum=" + "\"" + seqNum + "\"" + " shotId=" + "\"" + shotId + "\"/>");
			    }
			}
			writer.println("    "+"</videoSearchTopicResult>");
		       
		    }
		}
		writer.println("  </videoSearchRunResult>");
	    }
	    writer.println("</videoSearchResults>");

	    closeRunOutputFile();
	    if (runs.getLength() == 0) {
		System.err.println("Error: no <videoSearchRunResult> tag found.");
		System.err.print("Improper document formatting");
		System.err.println("");
		System.exit(-1);
	    }
	}
	catch (NullPointerException npe) {
	    System.err.println("Document not formatted properly; check run "+runIndex+".");
	    npe.printStackTrace();
	    System.exit(-7);
	}
    }




    
    /** Get an attribute from a given attributes list.  If the
     * attribute is not present, output an error message.*/
    public static String getAttribute(String attr_name, NamedNodeMap attributes, 
				      int runIndex, String sysid, 
				      String currResult, String currItem) {
	    
	Node attr = attributes.getNamedItem(attr_name);
	if (attr == null) {
	    if (sysid == null)
		System.err.println("Error: Cannot find sysid in run indexed at "+runIndex);
	    else if (currResult == null)
		System.err.println("Error: Cannot find attribute '"+attr_name+"' in "+sysid);
	    else if (currItem == null)
		System.err.println("Error: Cannot find attribute '"+attr_name+"' in result "+
				   currResult+" of "+sysid);
	    else
		System.err.println("Error: Cannot find attribute '"+attr_name+"' in item "+
				   currItem+" of result "+
				   currResult+" of "+sysid);
	}
	String attr_value = attr.getNodeValue();
	return attr_value;
    }


    /** Opens an output file.  Uses all six arguments for search, but
     * only trtype, sysid, priority (and outputPath) for feature.  */
    public static void makeNewRunOutputFile(String outputPath, String outFileName) { 

	try {
	    writer = new PrintWriter(new FileWriter(new File(outputPath, outFileName)));
	}
	catch (IOException e) {
	    System.err.println("Cannot open writer on "+outputPath+"/"+outFileName);
	    e.printStackTrace();
	    System.exit(0);
	}
    }

    /** Writes a treceval formatted line to the currently opened result file.
     */
    public static void writeResultLine(String num, String shotid, String seqnum, String sysid) {
	int comp = 10000 - Integer.parseInt(seqnum);
	writer.println(num+" 0 "+shotid+" "+seqnum+" "+comp+" "+sysid);
    }

    /** Close the current output file.
     */
    public static void closeRunOutputFile() {
	writer.flush();
	writer.close();
    }
}


/** The XML parser does not store child nodes in the order in which
 * they occur in the document.  This class provides an ordered node
 * list.  It also removes all nodes from the list which do not contain
 * the ordering String.
 */
class OrderedNodeList implements NodeList {
    
    private LinkedList nodes;
    
    OrderedNodeList(NodeList unordNodes, String orderAttribute) {
	nodes = new LinkedList();
	add(unordNodes,orderAttribute);
    }
    
    /** Add a NodeList to this OrderedNodeList, ordering them
     * according to the given orderAttribute.
     */
    void add(NodeList unordNodes, String orderAttribute) {
	for (int i = 0; i < unordNodes.getLength(); i++) {
	    if (unordNodes.item(i).hasAttributes() && 
		unordNodes.item(i).getAttributes().getNamedItem(orderAttribute) != null) {
		add(unordNodes.item(i),orderAttribute);
	    }
	}
    }
    
    /**
       Add a node to this OrderedNodeList, inserting it in the proper
       order according to the given orderAttribute.
     */
    void add(Node n, String orderAttribute) {
	if (nodes.size() == 0)
	    nodes.add(0, (Object)n);
	else {
	    String rank = n.getAttributes().getNamedItem(orderAttribute).getNodeValue();
	    int i = 0;
	    String currRank;
	    do {
		currRank = 
		    ((Node)nodes.get(i)).getAttributes().getNamedItem(orderAttribute).getNodeValue();
	    } while (rank.compareToIgnoreCase(currRank) > 0 && ++i < nodes.size());
	    nodes.add(i,(Object)n);
	}
    }
    
    public int getLength() {
	return nodes.size();
    }
    
    public Node item(int index) {
	return (Node)(nodes.get(index));
    }
}
